/**
 * @copyright 2015 Prometheus Research, LLC
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createDecoratedClass = (function () { function defineProperties(target, descriptors, initializers) { for (var i = 0; i < descriptors.length; i++) { var descriptor = descriptors[i]; var decorators = descriptor.decorators; var key = descriptor.key; delete descriptor.key; delete descriptor.decorators; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor || descriptor.initializer) descriptor.writable = true; if (decorators) { for (var f = 0; f < decorators.length; f++) { var decorator = decorators[f]; if (typeof decorator === 'function') { descriptor = decorator(target, key, descriptor) || descriptor; } else { throw new TypeError('The decorator for method ' + descriptor.key + ' is of the invalid type ' + typeof decorator); } } if (descriptor.initializer !== undefined) { initializers[key] = descriptor; continue; } } Object.defineProperty(target, key, descriptor); } } return function (Constructor, protoProps, staticProps, protoInitializers, staticInitializers) { if (protoProps) defineProperties(Constructor.prototype, protoProps, protoInitializers); if (staticProps) defineProperties(Constructor, staticProps, staticInitializers); return Constructor; }; })();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.createStylesheet = createStylesheet;
exports.isValidStylesheet = isValidStylesheet;
exports.overrideStylesheet = overrideStylesheet;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _memoizeDecorator = require('memoize-decorator');

var _memoizeDecorator2 = _interopRequireDefault(_memoizeDecorator);

var _styleLoaderAddStyles = require('style-loader/addStyles');

var _styleLoaderAddStyles2 = _interopRequireDefault(_styleLoaderAddStyles);

var _reactLibCSSPropertyOperations = require('react/lib/CSSPropertyOperations');

var _reactLibCSSPropertyOperations2 = _interopRequireDefault(_reactLibCSSPropertyOperations);

var _reactLibDangerousStyleValue = require('react/lib/dangerousStyleValue');

var _reactLibDangerousStyleValue2 = _interopRequireDefault(_reactLibDangerousStyleValue);

var _Utils = require('./Utils');

var SELF = 'self';

var DEFAULT_STYLE = {
  boxSizing: 'border-box'
};

var SUPPORTED_PSEUDO_CLASSES = {
  focus: true,
  hover: true,
  active: true,
  checked: true,
  'default': true,
  disabled: true,
  empty: true,
  enabled: true,
  firstChild: true,
  fullscreen: true,
  indeterminate: true,
  invalid: true,
  lastChild: true,
  left: true,
  link: true,
  onlyChild: true,
  optional: true,
  required: true,
  right: true,
  root: true,
  scope: true,
  target: true,
  valid: true,
  visited: true
};

function createStylesheet(spec) {
  var id = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];

  id = (0, _Utils.uniqueID)(id ? 'Style_' + id : 'Style');
  return new DOMStylesheet(convertSpecToStyle(spec), id);
}

function isValidStylesheet(obj) {
  return obj instanceof DOMStylesheet;
}

function overrideStylesheet(stylesheet, spec, id) {
  var style = isValidStylesheet(spec) ? spec.style : convertSpecToStyle(spec);
  var nextStyle = _extends({}, stylesheet.style);
  for (var key in style) {
    if (!style.hasOwnProperty(key)) {
      continue;
    }
    if (key === SELF) {
      nextStyle[key] = _extends({}, nextStyle[key], style[key]);
    } else {
      var value = style[key];
      nextStyle[key] = _extends({}, nextStyle[key]);
      for (var sKey in value) {
        if (!value.hasOwnProperty(sKey)) {
          continue;
        }
        nextStyle[key][sKey] = _extends({}, value[sKey]);
      }
    }
  }
  id = (0, _Utils.uniqueID)(id ? 'Style_' + id : 'Style');
  return new DOMStylesheet(nextStyle, id);
}

var DOMStylesheet = (function () {
  function DOMStylesheet(style, id) {
    _classCallCheck(this, DOMStylesheet);

    this.style = style;
    this.id = id;

    this._refs = 0;
    this._remove = null;
    this._disposePerform = this._disposePerform.bind(this);
    this._disposeTimer = null;
  }

  _createDecoratedClass(DOMStylesheet, [{
    key: 'asClassName',
    value: function asClassName() {
      var state = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var className = [];
      for (var key in this.className) {
        if (!this.className.hasOwnProperty(key)) {
          continue;
        }
        if (key === SELF || state[key]) {
          className.push(this.className[key]);
        }
      }
      return className.join(' ');
    }
  }, {
    key: 'use',
    value: function use() {
      this._refs = this._refs + 1;
      if (this._disposeTimer !== null) {
        clearTimeout(this._disposeTimer);
        this._disposeTimer = null;
      }
      if (this._remove === null) {
        this._remove = (0, _styleLoaderAddStyles2['default'])(this.css);
      }
      return this;
    }
  }, {
    key: 'dispose',
    value: function dispose() {
      this._refs = this._refs - 1;
      if (this._disposeTimer === null) {
        this._disposeTimer = setTimeout(this._disposePerform, 0);
      }
      return this;
    }
  }, {
    key: '_disposePerform',
    value: function _disposePerform() {
      if (this._refs < 1) {
        this._remove();
        this._remove = null;
      }
    }
  }, {
    key: '_compiled',
    decorators: [_memoizeDecorator2['default']],
    get: function get() {
      return compileStylesheet(this.style, this.id);
    }
  }, {
    key: 'css',
    get: function get() {
      return this._compiled.css;
    }
  }, {
    key: 'className',
    get: function get() {
      return this._compiled.className;
    }
  }]);

  return DOMStylesheet;
})();

function convertValue(key, value) {
  if ((0, _Utils.isArray)(value) && value.length > 0) {
    var rest = value.slice(1).map(function (v) {
      return key + ':' + (0, _reactLibDangerousStyleValue2['default'])(key, v);
    });
    value = [(0, _reactLibDangerousStyleValue2['default'])(key, value[0])].concat(rest).join(';');
  }
  return value;
}

function convertSpecToStyle(spec) {
  var addDefaultStyle = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];
  var recurse = arguments.length <= 2 || arguments[2] === undefined ? true : arguments[2];

  var style = _defineProperty({}, SELF, addDefaultStyle ? _extends({}, DEFAULT_STYLE) : {});

  for (var key in spec) {
    if (!spec.hasOwnProperty(key)) {
      continue;
    }
    var value = spec[key];
    if ((0, _Utils.isPlainObject)(value)) {
      if (recurse) {
        style[key] = convertSpecToStyle(value, false, false);
      } else {
        style[key] = convertValue(key, value);
      }
    } else {
      style[SELF][key] = convertValue(key, value);
    }
  }

  return style;
}

function compileStylesheet(style, id) {
  var mapping = {};
  var compiled = [];

  for (var key in style) {
    if (!style.hasOwnProperty(key)) {
      continue;
    }
    var value = style[key];

    var css = key === SELF ? _reactLibCSSPropertyOperations2['default'].createMarkupForStyles(value) : _reactLibCSSPropertyOperations2['default'].createMarkupForStyles(value[SELF]);

    if (SUPPORTED_PSEUDO_CLASSES[key]) {
      compiled.push(compilePseudoClass(mapping, css, key, id));
    } else {
      compiled.push(compileClass(mapping, css, key, id));
    }
    if (key !== SELF) {
      for (var sKey in value) {
        if (!value.hasOwnProperty(sKey)) {
          continue;
        }
        var sValue = value[sKey];
        if (sKey === SELF || !SUPPORTED_PSEUDO_CLASSES[sKey]) {
          continue;
        }
        var _css = _reactLibCSSPropertyOperations2['default'].createMarkupForStyles(sValue);
        compiled.push(compilePseudoClass(mapping, _css, sKey, id + '--' + key, false));
      }
    }
  }

  compiled = compiled.join('\n');

  return { css: [[id, compiled]], className: mapping };
}

function compileClass(mapping, css, name, id) {
  var className = name === SELF ? id : id + '--' + name;
  mapping[name] = className;
  return '.' + className + ' { ' + css + ' }';
}

function compilePseudoClass(mapping, css, name, id) {
  var compileAsRegular = arguments.length <= 4 || arguments[4] === undefined ? true : arguments[4];

  var pseudoClassName = id + ':' + (0, _Utils.toDashCase)(name);
  if (compileAsRegular) {
    var className = id + '--' + name;
    mapping[name] = className;
    return '.' + className + ', .' + pseudoClassName + ' { ' + css + ' }';
  } else {
    return '.' + pseudoClassName + ' { ' + css + ' }';
  }
}
